using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;


namespace AcidRainTutorialNew
{
    /// <summary>
    /// This is a game component that implements IUpdateable.
    /// </summary>
    public class FrontOfPlayer : Microsoft.Xna.Framework.DrawableGameComponent
    {
        //Declare a spritebatch to be used in this class
        //We will retrieve this from the game services collection in the constructor
        SpriteBatch spriteBatch;
        ContentManager content;
        Texture2D mapObjectsSheet;

        #region Front of Player Objects (Rocks) Struct and Variables
        public struct MapObject
        {
            public Rectangle SourceRect;
            public Rectangle CollisionRect;
            public Vector2 Position;
            public SpriteEffects Flip;
        }

        public MapObject[] frontObjects;
        public int numFrontObjects = 30;
        #endregion
       
        #region Tree Struct and Variables
        //create object using struct, trees
        public struct TreesStruct
        {
            public Texture2D TreeTexture;
            public float TreeWidth;
            public float TreeHeight;
            public Rectangle TreeSourceRect;
            public Texture2D TrunkTexture;
            public float TrunkWidth;
            public float TrunkHeight;
            public Rectangle TrunkSourceRect;
            public bool TrunkShadow;
            public Rectangle TreeRect;
            public Rectangle CanopyCollisionRect;
            public bool TreeCollision;
            public Rectangle ShelterRect;
            public bool TreeIsBurning;
            public int TreeBurnTime;
            public int BurnTreeType;
        }

        public static TreesStruct[] Trees;  //static, make available to all classes for Collision Detection
        private int numOfTrees = 200;
        Rectangle burntTree2SourceRect;
      
        Rectangle burntTree2TrunkSourceRect;
    
        Rectangle burntTree6SourceRect;
   
        #endregion

        public FrontOfPlayer(Game game)
            : base(game)
        {
            Trees = new TreesStruct[numOfTrees];
            frontObjects = new MapObject[numFrontObjects];
            
            //Retrieve from Game Services
            spriteBatch = ((SpriteBatch)game.Services.GetService(typeof(SpriteBatch)));
            content = ((ContentManager)game.Services.GetService(typeof(ContentManager)));
         }
        
        public override void Initialize()
        {
           base.Initialize();
        }

        protected override void LoadContent()
        {
    
            mapObjectsSheet = content.Load<Texture2D>(@"Background/Map/MapObjectsSheetTutorial");

            #region Make Rocks
            for (int i = 0; i < frontObjects.Length; i++)
            {
                frontObjects[i].Flip = SpriteEffects.None;

                switch (Engine.Random.Next(2))
                {

                    //Rock1
                    case 0:
                        frontObjects[i].SourceRect = new Rectangle(769, 142, 255, 108);
                        break;

                    //Rock3
                    case 1:
                        frontObjects[i].SourceRect = new Rectangle(769, 255, 255, 100);
                        break;
                    //default:
                }
                //Position Rocks so they are between mountains, on map
                frontObjects[i].Position = new Vector2(Engine.Random.Next(1600, (int)(Terrain.TotalMapWidth - (frontObjects[i].SourceRect.Width + 1200))),
                                                       Engine.Random.Next(300, (int)(Terrain.TotalMapHeight - (frontObjects[i].SourceRect.Height + 300))));

                frontObjects[i].CollisionRect = new Rectangle((int)frontObjects[i].Position.X, (int)frontObjects[i].Position.Y + 65,
                                                               frontObjects[i].SourceRect.Width, frontObjects[i].SourceRect.Height - 80);

            }
            #endregion
           
            #region Make Trees
            //canopyTopBurnt = content.Load<Texture2D>(@"Background/Trees/BurntTree2Top");
            burntTree2SourceRect = new Rectangle(1024, 720, 264, 230); //BurntTree2Top
    
            //canopyTrunkBurnt = content.Load<Texture2D>(@"Background/Trees/BurntTree2Trunk");
            burntTree2TrunkSourceRect = new Rectangle(1287, 719, 32, 112); //BurntTrunk2Top
      

            //skinnyTreeBurnt = content.Load<Texture2D>(@"Background/Trees/BurntTree6");
            burntTree6SourceRect = new Rectangle(1652, 360, 256, 420); //BurntTree6
       
            //Make Trees
            for (int i = 0; i < Trees.Length; i++)
            {
                Trees[i].TreeIsBurning = false;  //Start with no trees burning
                Trees[i].TreeBurnTime = 200; //Set timer for tree burning time
                Trees[i].TrunkShadow = false;
                Trees[i].BurnTreeType = 0;

                switch (Engine.Random.Next(4))
                {
                    case 0:
                        Trees[i].TreeSourceRect = new Rectangle(1024, 0, 256, 360); //Canopy145
                        Trees[i].TreeWidth = 256;
                        Trees[i].TreeHeight = 360;

                        Trees[i].TrunkSourceRect = new Rectangle(1280, 0, 176, 304); //Trunk145
                        Trees[i].TrunkWidth = 176;
                        Trees[i].TrunkHeight = 304;
                        break;

                    case 1:
                        Trees[i].TreeSourceRect = new Rectangle(1456, 0, 256, 328); //Fur2Canopy
                        Trees[i].TreeWidth = 256;
                        Trees[i].TreeHeight = 328;

                        Trees[i].TrunkSourceRect = new Rectangle(1712, 0, 92, 168); //Fur2Trunk
                        Trees[i].TrunkWidth = 92;
                        Trees[i].TrunkHeight = 168;
                        break;

                    case 2:

                        Trees[i].TreeSourceRect = new Rectangle(1024, 360, 372, 360); //LeafTreeCanopy400
                        Trees[i].TreeWidth = 372;
                        Trees[i].TreeHeight = 360;

                        Trees[i].TrunkSourceRect = new Rectangle(1803, 0, 84, 100); //LeafTreeTrunk
                        Trees[i].TrunkWidth = 84;
                        Trees[i].TrunkHeight = 100;
                        Trees[i].TrunkShadow = true;
                        break;

                    case 3:

                        Trees[i].TreeSourceRect = new Rectangle(1396, 360, 256, 420); //SkinnyTree256
                        Trees[i].TreeWidth = 256;
                        Trees[i].TreeHeight = 420;

                        Trees[i].TrunkSourceRect = new Rectangle(2046, 0, 1, 1); //Transparent
                        Trees[i].TrunkWidth = 1;
                        Trees[i].TrunkHeight = 1;
                        Trees[i].BurnTreeType = 1;
                        break;
                    //default:
                }
                //TreeRect is the position of our tree.
                Trees[i].TreeRect = new Rectangle(Engine.Random.Next(1600, (Terrain.TotalMapWidth - (int)Trees[i].TreeWidth) - 1200),
                                                         Engine.Random.Next(200, (Terrain.TotalMapHeight - (int)Trees[i].TreeHeight) - 300),
                                                         (int)Trees[i].TreeWidth,
                                                         (int)Trees[i].TreeHeight);
                //This is the Bottom area of the tree, used for people to seek shelter in person AI routine
                Trees[i].ShelterRect = new Rectangle((int)(Trees[i].TreeRect.X + (Trees[i].TreeRect.Width * .48)),
                                                    (int)(Trees[i].TreeRect.Y + (Trees[i].TreeRect.Height * .8)),
                                                    (int)(Trees[i].TreeRect.Width * .1), (int)(Trees[i].TreeRect.Height * .1));
                //Use this for better looking AcidRain collisions
                Trees[i].CanopyCollisionRect = new Rectangle((int)(Trees[i].TreeRect.X + (Trees[i].TreeRect.Width * .3)),
                                                           (int)(Trees[i].TreeRect.Y + (Trees[i].TreeRect.Height * .3)),
                                                           (int)(Trees[i].TreeRect.Width * .5), (int)(Trees[i].TreeRect.Height * .5));

                Trees[i].TreeCollision = true;
            }
            #endregion

            base.LoadContent();
        }

        public override void Update(GameTime gameTime)
        {
   
            base.Update(gameTime);
        }

        public override void Draw(GameTime gameTime)
        {
      
            spriteBatch.Begin(SpriteBlendMode.AlphaBlend, SpriteSortMode.Immediate, SaveStateMode.None, ScreenManager.Camera.Transform);

            #region Draw FrontGround Objects (Rocks)
            //Draw Background Objects
            for (int i = 0; i < frontObjects.Length; i++)
            {
                //spriteBatch.Draw(testRect, frontObjects[i].CollisionRect, Color.White);
                spriteBatch.Draw(mapObjectsSheet, frontObjects[i].Position, frontObjects[i].SourceRect,
                                        Color.White, 0f, Vector2.One, 1f,
                                        frontObjects[i].Flip, 0f);
            }
            #endregion

            #region Draw Trees
            //2 draws for each tree.  The first is the object and the second is the shadow.  The shadow is just the original object
            //with color set to black with transparency set and adjusted position, rotated around origin point
            for (int i = 0; i < Trees.Length; i++)
            {
                //Tree Shadow
                spriteBatch.Draw(mapObjectsSheet,
                                 new Vector2(Trees[i].TreeRect.Center.X,
                                 Trees[i].TreeRect.Bottom),
                                 Trees[i].TreeSourceRect,
                                 new Color(0, 0, 0, 100),
                                 MathHelper.ToRadians(15),
                                 new Vector2(Trees[i].TreeWidth / 2, Trees[i].TreeHeight),
                                 1f,
                                 SpriteEffects.None, 0f);

                //Tree Texture
                spriteBatch.Draw(mapObjectsSheet,
                                Trees[i].TreeRect,
                                Trees[i].TreeSourceRect,
                                Color.White,
                                0f,
                                new Vector2(0f, 0f),
                                SpriteEffects.None, 0f);

             
            }
            #endregion

            spriteBatch.End();
            
            base.Draw(gameTime);
        }

    }
}