﻿using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;
using System.Text;

namespace AcidRainTutorialNew
{
    public class Player : Microsoft.Xna.Framework.DrawableGameComponent
    {
        GameplayScreen gamePlayScreen;
        ScreenManager screenManager;
        private Rectangle screenBounds;
        SpriteBatch spriteBatch;
        ContentManager content;
        Texture2D mapObjectsSheet;
        SpriteFont font;

        #region Player Variables
        Texture2D playerTex;
        public static Vector2 PlayerPos;
        private Vector2 startPlayerPos;
        public Vector2 playerSpeed;
        float maxPlayerSpeed = 7f; //7 If you increase maxspeed, you need to decrease runFramerateFactor (frameRate = playerSpeed.X * 7f;)
        #endregion

        #region Controls
        float playerDegrees;
        GamePadState gamepad;  //Joystick object labeled
        KeyboardState keyboard; //Keyboard object labeled
        Vector2 leftStick;
        public PlayerIndex playerIndex;
        #endregion

        public Player(Game game, GameplayScreen gamePlayScreen, ScreenManager screenManager, PlayerIndex playerIndex)
            : base(game)
        {
            this.gamePlayScreen = gamePlayScreen;
            this.screenManager = screenManager;
            this.playerIndex = playerIndex;

            //Retrieve from Game Services
            spriteBatch = ((SpriteBatch)game.Services.GetService(typeof(SpriteBatch)));
            content = ((ContentManager)game.Services.GetService(typeof(ContentManager)));

            PlayerPos = GameplayScreen.PlayerStartPos;
            startPlayerPos = PlayerPos;
   
        }

        public override void Initialize()
        {
             base.Initialize();
        }
        
        protected override void LoadContent()
        {
     
            font = content.Load<SpriteFont>(@"Fonts/DayRoman36");

            //ScreenBounds is the viewport size as defined in Engine
            screenBounds = new Rectangle(0, 0, Game.GraphicsDevice.Viewport.Width, Game.GraphicsDevice.Viewport.Height);
            mapObjectsSheet = content.Load<Texture2D>(@"Background/Map/MapObjectsSheetTutorial");
            playerTex = content.Load<Texture2D>(@"PlayerTextures/Right");
            base.LoadContent();
        }

        //Detrmine if user has joystick or keyboard and then call appropriate methods
        public override void Update(GameTime gameTime)
        {
            float elapsed = (float)gameTime.ElapsedGameTime.TotalSeconds;
 
            GetInput(elapsed);      //Get Keyboard and Joystick input, Set AnimationState to walking, running or Idle
            CameraPlayerAttach();   //Attach Camera to Player
            CameraScreenBoundry();  //Check screen boundry and stop camera movement if hit bounds.
            PlayerScreenBoundry();
 
            base.Update(gameTime);
         }

        public void GetInput(float elapsed)
        {
            #region Joystick
            if (GamePad.GetState((PlayerIndex)playerIndex).IsConnected)                          //Is Joystick COnnected?
            {
                gamepad = GamePad.GetState((PlayerIndex)playerIndex);                           //Instantiate gamepad
                leftStick = GamePad.GetState((PlayerIndex)playerIndex).ThumbSticks.Left; //Make Leftstick a Vector

                if (leftStick.Length() > .2f)           //Player must move stick to at least this value, set low value in case of loose joysticks
                {
                    if (leftStick.Length() > .90f)      //If stick has almost full extension, playerState is running
                    {
                        //Use gametime if we notice any jitter in player movement
                        //playerSpeed = new Vector2(maxPlayerSpeed * 60 * elapsed, maxPlayerSpeed * 60 * elapsed);
                        playerSpeed = new Vector2(maxPlayerSpeed, maxPlayerSpeed);  //Set player speed to Max
 

                    }
                    else //Walking
                    {
                        //Adjust playerspeed by position of stick..note maxplayerspeed is multiplied by the length of stick
                        //for each direction...X and Y and this becomes playerspeed
                        playerSpeed = new Vector2(leftStick.Length() * maxPlayerSpeed, leftStick.Length() * maxPlayerSpeed);
                     }

                    PlayerPos += new Vector2(playerSpeed.X * gamepad.ThumbSticks.Left.X, playerSpeed.Y * gamepad.ThumbSticks.Left.Y * -1);//Move player
                }
            }
            else
            {   //If Joystick is not connected, use keyboard
                keyboard = Keyboard.GetState();  //Create keyboard object...Instantiate
                Vector2 motion = Vector2.Zero;   //Create vector to be used in determining angle   

                if (keyboard.IsKeyDown(Keys.LeftShift) || keyboard.IsKeyDown(Keys.RightShift))
                {   //Running

                    playerSpeed = new Vector2(maxPlayerSpeed, maxPlayerSpeed);  //Set player speed to Max
                }
                else
                {   //Walking

                    playerSpeed = new Vector2(maxPlayerSpeed / 2, maxPlayerSpeed / 2);  //Set player speed to Max
                }

                //Basic movement depending on keys pressed
                if (keyboard.IsKeyDown(Keys.W))
                    motion.Y--;
                if (keyboard.IsKeyDown(Keys.S))
                    motion.Y++;
                if (keyboard.IsKeyDown(Keys.A))
                    motion.X--;
                if (keyboard.IsKeyDown(Keys.D))
                    motion.X++;

                //normalize evens out the movement for the diagonal direction.  Without this, diagonal movement is faster because a diagonal 
                //line is a shorter distance than a straight line.  You only have this problem with keyboard movement.
                //Convert X and Y directions into angle.  This allows you to determine direction based upon degrees around a half circle.
                //See UpdatePlayerAnimation method in this class for explanation.
                if (motion != Vector2.Zero)
                {
                    motion.Normalize();
                    PlayerPos += motion * playerSpeed.X;
                    float keyboardAngle = (float)Math.Atan2(-motion.Y, motion.X);   //Atan2 converts X,Y to an angle
                    playerDegrees = MathHelper.ToDegrees(keyboardAngle);            //Convert Angle in radians to degrees
                }
            }
                #endregion
        }

        public override void Draw(GameTime gameTime)
        {
           
            spriteBatch.Begin(SpriteBlendMode.AlphaBlend, SpriteSortMode.Immediate, SaveStateMode.None, ScreenManager.Camera.Transform);
            spriteBatch.Draw(playerTex, PlayerPos, null, Color.White, 0f, new Vector2(90, 90), 1f, SpriteEffects.None, 0f);
            spriteBatch.End();
            
            base.Draw(gameTime);
        }

        private void CameraPlayerAttach()
        {
            //Attach camera to player so camera moves with player.  Camera is set at 0,0 in camera class.
            //Player Origin sets player position in the center of the player texture/animation.
            //First we make camera position = to player position.  Then we need to subtract half the screen width and height
            //So the camera position ends up at the top left of the screen.
            //We adjust the screen width and height by the camera scale by dividing.  This keeps the player in the center of the screen.
            //So as the camera scales, we divide screen dimensions by camera scale.
            ScreenManager.Camera.Position.X = Player.PlayerPos.X - ((screenBounds.Width / 2) / ScreenManager.Camera.scale);
            ScreenManager.Camera.Position.Y = Player.PlayerPos.Y - ((screenBounds.Height / 2) / ScreenManager.Camera.scale);
        }
        //Check screen boundry and stop camera movement if hit bounds.  Except for Top.
        //I want the player to be able to see above the map so they can see the sky and the top of the mountains.
        private void CameraScreenBoundry()
        {
            if (ScreenManager.Camera.Position.X < screenBounds.Left / ScreenManager.Camera.scale)
                ScreenManager.Camera.Position.X = screenBounds.Left / ScreenManager.Camera.scale;
            //if (ScreenManager.Camera.Position.Y < screenBounds.Top / ScreenManager.Camera.scale)
            //    ScreenManager.Camera.Position.Y = screenBounds.Top / ScreenManager.Camera.scale;

            if (ScreenManager.Camera.Position.X > Terrain.TotalMapWidth - screenBounds.Width / ScreenManager.Camera.scale)
                ScreenManager.Camera.Position.X = Terrain.TotalMapWidth - screenBounds.Width / ScreenManager.Camera.scale;
            if (ScreenManager.Camera.Position.Y > Terrain.TotalMapHeight - screenBounds.Height / ScreenManager.Camera.scale)
                ScreenManager.Camera.Position.Y = Terrain.TotalMapHeight - screenBounds.Height / ScreenManager.Camera.scale;
        }

        private void PlayerScreenBoundry()
        {
            //Keep the player inside the screen.  Check bounderies of map using screenbounds on the left and Top
            //and totalmap on the  bottom and right.  Adjust for player texture dimensions.
            if (Player.PlayerPos.X - playerTex.Width / 4 < screenBounds.Left)
            { Player.PlayerPos.X = screenBounds.Left + playerTex.Width / 4; }
            if (Player.PlayerPos.Y - playerTex.Width / 4 < screenBounds.Top)
            { Player.PlayerPos.Y = screenBounds.Top + playerTex.Width / 4; }
            if (Player.PlayerPos.Y + playerTex.Width / 4 > Terrain.TotalMapHeight)
                Player.PlayerPos.Y = Terrain.TotalMapHeight - playerTex.Width / 4;
            if (Player.PlayerPos.X + playerTex.Width / 4 > Terrain.TotalMapWidth)
                Player.PlayerPos.X = Terrain.TotalMapWidth - playerTex.Width / 4;
        }
    }
 }
