using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;
using System.IO;

namespace AcidRainTutorialNew
{
    public class Terrain : Microsoft.Xna.Framework.DrawableGameComponent
    {
        //Declare a spritebatch to be used in this class
        //We will retrieve this from the game services collection in the constructor
        SpriteBatch spriteBatch;
        ContentManager content;
        Player player;
        Rectangle titleSafeArea;
        SpriteFont font;
         private Texture2D mapObjectsSheet;

        #region Setup variables for Ground Tiles
        int mapWidth = 20;
        int mapHeight = 10;
        private Rectangle groundSourceBase;
        public static int TotalMapWidth;
        public static int TotalMapHeight;
        #endregion

        #region Map Objects Struct
        public struct MapObject
        {
            public Rectangle SourceRect;
            public Rectangle CollisionRect;
            public Vector2 Position;
            public float Scale;
            public SpriteEffects Flip;
        }

        public MapObject[] backObjects;
        private int numBackObjects = 200;
        #endregion

        public Terrain(Game game, Player player)
            : base(game)
        {
            this.player = player;

            backObjects = new MapObject[numBackObjects];    //From BackOfPlayer

            //Retrieve from Game Services
            spriteBatch = ((SpriteBatch)game.Services.GetService(typeof(SpriteBatch)));
            content = ((ContentManager)game.Services.GetService(typeof(ContentManager)));
        }

         public override void Initialize()
        {
            groundSourceBase = new Rectangle(0, 0, 512, 512);
            TotalMapWidth = groundSourceBase.Width * mapWidth;
            TotalMapHeight = groundSourceBase.Height * mapHeight;
            
            base.Initialize();
        }

         protected override void LoadContent()
         {
             font = content.Load<SpriteFont>(@"Fonts/DayRoman36");
             titleSafeArea = Game.GraphicsDevice.Viewport.TitleSafeArea;
             mapObjectsSheet = content.Load<Texture2D>(@"Background/Map/MapObjectsSheetTutorial");

             #region Make Main Map Background Objects
             for (int i = 0; i < backObjects.Length; i++)
             {
                 backObjects[i].Flip = SpriteEffects.None;
                 backObjects[i].Scale = 1f;

                 int weight = Engine.Random.Next(100);
                 if (weight > 20)
                 {
                     switch (Engine.Random.Next(6))
                     {
                         //GroundLump1
                         case 0:
                             backObjects[i].SourceRect = new Rectangle(514, 0, 255, 140);
                             break;
                         //GroundLump2
                         case 1:
                             backObjects[i].SourceRect = new Rectangle(769, 0, 255, 140);
                             break;
                         //Snow1
                         case 2:
                             backObjects[i].SourceRect = new Rectangle(856, 360, 174, 152);
                             backObjects[i].Flip = SpriteEffects.None;
                             backObjects[i].Scale = Engine.RandomBetween(.7f, 2f);
                             break;
                         //Snow2
                         case 3:
                             backObjects[i].SourceRect = new Rectangle(514, 724, 514, 300);
                             backObjects[i].Flip = SpriteEffects.FlipHorizontally;
                             backObjects[i].Scale = Engine.RandomBetween(.7f, 2f);
                             break;
                         //Snow3
                         case 4:
                             backObjects[i].SourceRect = new Rectangle(511, 512, 513, 202);
                             backObjects[i].Flip = SpriteEffects.None;
                             backObjects[i].Scale = Engine.RandomBetween(.7f, 2f);
                             break;
                         //Snow4
                         case 5:
                             backObjects[i].SourceRect = new Rectangle(0, 718, 514, 306);
                             backObjects[i].Flip = SpriteEffects.FlipHorizontally;
                             backObjects[i].Scale = Engine.RandomBetween(.7f, 2f);
                             break;
                     }
                 }
                 else
                 {
                     switch (Engine.Random.Next(4))
                     {
                         //DownTree
                         case 0:
                             backObjects[i].SourceRect = new Rectangle(514, 142, 255, 108);
                             //backObjects[i].Depth = objectDepth;
                             break;
                         //DownRock1
                         case 1:
                             backObjects[i].SourceRect = new Rectangle(514, 360, 350, 152);
                             //backObjects[i].Depth = objectDepth;
                             break;
                         //DownRock2
                         case 2:
                             backObjects[i].SourceRect = new Rectangle(0, 517, 510, 200);
                             //backObjects[i].Depth = objectDepth;
                             break;

                         //Rock2
                         case 3:
                             backObjects[i].SourceRect = new Rectangle(514, 255, 255, 100);
                             //backObjects[i].Depth = objectDepth;
                             break;

                         //default:
                     }
                 }
                 backObjects[i].Position = new Vector2(Engine.Random.Next(1600, //width of mountains 
                                                     (int)(Terrain.TotalMapWidth - (backObjects[i].SourceRect.Width * backObjects[i].Scale) - 1200)),
                                                      Engine.Random.Next(300, (int)(Terrain.TotalMapHeight -
                                                     (backObjects[i].SourceRect.Height * backObjects[i].Scale) - 300)));
             }
             #endregion

             base.LoadContent();
         }

         public override void Update(GameTime gameTime)
         {
             base.Update(gameTime);
         }

        public override void Draw(GameTime gameTime)
         {
             spriteBatch.Begin(SpriteBlendMode.AlphaBlend, SpriteSortMode.Immediate, SaveStateMode.None, ScreenManager.Camera.Transform);

             for (int x = 0; x < mapWidth; x++)
             {
                 for (int y = 0; y < mapHeight; y++)
                 {  //Draw ground base texture across entire map, use 512 X 512 incremented by for loops
                     //groundSourceBase = the main texture at position 0,0, 512 width, 512 height.
                     spriteBatch.Draw(mapObjectsSheet, new Vector2(x * groundSourceBase.Width, y * groundSourceBase.Height),
                                      groundSourceBase, Color.White, 0f, Vector2.One, 1f, SpriteEffects.None, 1f);
                 }
             }
  
            //Draw Map Objects behind Player
            for (int i = 0; i < backObjects.Length; i++)
            {
                spriteBatch.Draw(mapObjectsSheet, backObjects[i].Position, backObjects[i].SourceRect,
                                        Color.White, 0f, Vector2.One, 1f,
                                        backObjects[i].Flip, 0f);
            }

            #region Draw Tree Trunks that are in back of player
            //Place bottom center of trunk texture into bottom center of canopy rectangle
            for (int i = 0; i < FrontOfPlayer.Trees.Length; i++)
            {
                //Trunk Shadow - Only draw a shadow for the LeafTreeTrunk. 
                if (FrontOfPlayer.Trees[i].TrunkShadow == true)
                {
                    spriteBatch.Draw(mapObjectsSheet,
                                 new Vector2(FrontOfPlayer.Trees[i].TreeRect.Center.X,
                                 FrontOfPlayer.Trees[i].TreeRect.Bottom),
                                 FrontOfPlayer.Trees[i].TrunkSourceRect,
                                 new Color(0, 0, 0, 100),
                                 MathHelper.ToRadians(15),
                                 new Vector2(FrontOfPlayer.Trees[i].TrunkWidth / 2, FrontOfPlayer.Trees[i].TrunkHeight),
                                 1f,
                                 SpriteEffects.None, 0f);
                }
                //Trunk Texture
                spriteBatch.Draw(mapObjectsSheet,
                            new Vector2(FrontOfPlayer.Trees[i].TreeRect.Center.X, FrontOfPlayer.Trees[i].TreeRect.Bottom),
                            FrontOfPlayer.Trees[i].TrunkSourceRect, Color.White, 0f,
                            new Vector2(FrontOfPlayer.Trees[i].TrunkWidth / 2, FrontOfPlayer.Trees[i].TrunkHeight),
                            1f, SpriteEffects.None, 0f);
            }
            #endregion

           spriteBatch.End();

             base.Draw(gameTime);
          }
    }
}

